/*
 * boot.c
 *
 * Last modified: <2009/11/29 23:08:53 +0900> By SUMIDA
 */
#include <SDL.h>
#include <SDL_mixer.h>
#include <stdlib.h>
#include <math.h>

#include "main.h"
#include "log.h"

static unsigned int video_mode = SDL_SWSURFACE|SDL_SWSURFACE|SDL_DOUBLEBUF;

/*****************************************************************************/
/* ϐ`                                                              */
/*****************************************************************************/
/* XN[T[tFX */
SDL_Surface *screen = NULL;

/* ޔXN[ */
SDL_Surface *backing = NULL;

/* t[JEg */
unsigned int frame_count = 0;

/*****************************************************************************/
/* ֐Q                                                                    */
/*****************************************************************************/
/*  */
int Initialize(void) {

	unsigned int rmask, gmask, bmask, amask;
	int i;

	REPORT("Initialize ... start.");

	/* SDL */
	REPORT("SDL_Init ... start.");
	if (SDL_Init(SDL_INIT_VIDEO
				 | SDL_INIT_AUDIO
				 | SDL_INIT_TIMER
				 | SDL_INIT_JOYSTICK) < 0) {

		ERROR(format("SDL̏Ɏs܂F%s", SDL_GetError()));
		return -1;
	}
	REPORT("SDL_Init ... done.");

	/* ʂ */
	REPORT("SDL_SetVideoMode ... start.");
	screen = SDL_SetVideoMode(WIDTH, HEIGHT, DEPTH,
							  /*SDL_FULLSCREEN|SDL_HWSURFACE|SDL_DOUBLEBUF/**/
							  /*SDL_SWSURFACE|SDL_SWSURFACE|SDL_DOUBLEBUF/**/
								  video_mode
							  );
	if (screen == NULL) {
		ERROR(format("ʂ̏Ɏs܂F%s", SDL_GetError()));
		SDL_Quit();
		return -1;
	}
	REPORT("SDL_SetVideoMode ... done.");

	/* ACRݒ肷 */
	SDL_WM_SetIcon(SDL_LoadBMP("icon.bmp"), NULL);

	/* t[T[tFX擾 */
	REPORT("SDL_CreateRGBSurface ... start.");
	rmask = 0x00ff0000;
	gmask = 0x0000ff00;
	bmask = 0x000000ff;
	amask = 0x00000000;

	backing = SDL_CreateRGBSurface(
		SDL_SWSURFACE, WIDTH, HEIGHT, DEPTH, rmask, gmask, bmask, amask);
	if(backing == NULL) {
		ERROR(format("CreateRGBSurface Ɏs܂: %s", SDL_GetError()));
		return -1;
	}
	REPORT("SDL_CreateRGBSurface ... done.");

#ifdef USE_AUDIO
	REPORT("Mix_OpenAudio ... start.");
	/*
	 * I[fBI
	 * open 44.1KHz, signed 16bit, system byte order,
	 *      stereo audio, using 4096 byte chunks
	 */
	if (Mix_OpenAudio(44100, MIX_DEFAULT_FORMAT, 2, 4096) < 0) {
		ERROR(format("I[fBIɎs܂F%s", SDL_GetError()));
		SDL_Quit();
		return -1;
	}
	REPORT("Mix_OpenAudio ... done.");
#endif

	/* ʂ̃^CgύX */
	SDL_WM_SetCaption(TITLE, NULL);
	REPORT("SDL_WM_SetCaption ... done.");

	if (video_mode & SDL_FULLSCREEN) {
		/* }EXJ[\\ɕύX */
		SDL_ShowCursor(SDL_DISABLE);
		REPORT("SDL_ShowCursor ... done.");
	}

	REPORT("Initialize ... done.");

	return 0;
}

/* `揈 */
void Draw(void) {

	if (SDL_MUSTLOCK(screen)) {
		if (SDL_LockSurface(screen) < 0) return;
	}

	/* XN[ɕ` */
	game_draw(screen->pixels);

	if (SDL_MUSTLOCK(screen)) {
		SDL_UnlockSurface(screen);
	}

	/* XN[ޔ */
	/*SDL_BlitSurface(screen, NULL, backing, NULL); /**/

	/* ʂXV */
	SDL_Flip(screen);
}

/* C[v */
void MainLoop(void) {

	SDL_Event event;
	double next_frame = SDL_GetTicks();
	double wait = 1000.0 / 30;
	int busy = 1;

	REPORT("MainLoop ... start.");

	for (;;) {
		/* ׂẴCxg */
		while (SDL_PollEvent(&event)) {
			/* QUIT Cxg邩AESC L[ꂽI */
			if ((event.type == SDL_QUIT) ||
				(event.type == SDL_KEYUP && event.key.keysym.sym == SDLK_ESCAPE)) {
				busy = 0;
				break;
			}

			/* NOTE : Cxghuȉʂ̏ꍇ́AōXVs */
		}

		if (!busy) break;

		/* 1bԂ60Update悤ɂ */
		if (SDL_GetTicks() >= next_frame) {

			/* XV̌Ăяo */
			game_update();

			/* Ԃ܂ƂDraw */
			// R}ƃQ[ɂȂ̂ŋ\c
			//if (SDL_GetTicks() < next_frame + wait)
				Draw();

			next_frame += wait;
			SDL_Delay(0);
		}

		++frame_count;
	}

	REPORT("MainLoop ... done.");
}

/* C֐ */
int SDL_main(int argc, char* argv[]) {

	int i;

	REPORT("SDL_main ... start.");

	/* R}hC̉ */
	for (i = 0; i < argc; i++) {
		REPORT(format("argv[%d]=%s", i, argv[i]));
		if (strncmp("-f", argv[i], 2) == 0) {
			video_mode = SDL_FULLSCREEN|SDL_HWSURFACE|SDL_DOUBLEBUF;
		}
	}

	/* SDL֘Ȁ */
	if (Initialize() < 0) return EXIT_FAILURE;

	/* ̌Ăяo */
	if (game_init() < 0) return EXIT_FAILURE;

	/* C[v */
	MainLoop();

	/* ǏĂяo */
	game_term();

	/* I */
	SDL_Quit();

	REPORT("SDL_main ... done.");

	return EXIT_SUCCESS;
}

