/*
 * font.c
 *
 * Last modified: <2009/06/20 14:44:23 +0900> By SUMIDA
 */
#include <stdio.h>
#include <stdarg.h>
#include <SDL.h>
#include "main.h"
#include "font.h"
#include "log.h"

typedef struct CONSOLE {

	int font_x;
	int font_y;
	int frame_x;
	int frame_y;
	int frame_w;
	int frame_h;
	unsigned char *map;
	SDL_Surface *surface;

} CONSOLE;

static int font_size_w;
static int font_size_h;
static int font_map_w;
static int font_map_h;
static CONSOLE console;

static char *strbuf = NULL;

void font_init(const char *filename, int font_w, int font_h) {

	size_t map_size;
	SDL_Surface *surface;
	SDL_Surface *temp;

	REPORT("font_init ... start.");

	memset(&console, 0, sizeof(console));

	font_size_w = font_w;
	font_size_h = font_h;
	font_map_w  = WIDTH  / font_size_w;
	font_map_h = HEIGHT / font_size_h;

	strbuf = malloc(sizeof(char) * font_map_w * font_map_h);
	map_size = sizeof(unsigned char) * font_map_w * font_map_h;

	console.font_x  = 0;
	console.font_y  = 0;
	console.frame_x = 0;
	console.frame_y = 0;
	console.frame_w = font_map_w;
	console.frame_h = font_map_h;
	console.map = malloc(map_size);
	memset(console.map, 0, map_size);

	REPORT(format("SDL_LoadBMP(%s)", filename));
	temp = SDL_LoadBMP(filename);

	if (temp == NULL) {
		ERROR(format("Couldn't load: %s", SDL_GetError()));

	} else {
#if 0
		/* (x,y)=(0,0)̐F𓧉ߐFƂ */
		Uint32 pixel = 0;
		memcpy(&pixel, temp->pixels, temp->format->BytesPerPixel);
		SDL_SetColorKey(temp, (SDL_SRCCOLORKEY|SDL_RLEACCEL), pixel);

		/* ݂̃rfItH[}bgɕϊ */
		surface = SDL_DisplayFormat(temp);
		SDL_FreeSurface(temp);
		if (temp == NULL) {
			ERROR(format("Couldn't convert sprite: %s", SDL_GetError()));

		} else {
			/* T[tFXo^ */
			console.surface = surface;
		}
#endif
		console.surface = temp;
	}
	REPORT("font_init ... done.");
}

void font_term(void) {

	REPORT("font_term ... start.");

	free(strbuf);
	strbuf = NULL;
	free(console.map);
	console.map = NULL;
	SDL_FreeSurface(console.surface);

	REPORT("font_term ... done.");
}

void font_draw(void) {

	int col, row;
	SDL_Rect rect, area;

	rect.w = font_size_w;
	rect.h = font_size_h;
	area.w = 0;
	area.h = 0;

	for (row = 0; row < font_map_h; row++) {
		int y = row * font_size_h;
		unsigned char *line = &console.map[row * font_map_w];

		for (col = 0; col < font_map_w; col++) {
			unsigned char attr = line[col];

			if (attr) {
				int x = col * font_size_w;

				int w = console.surface->w / font_size_w;
				int sx = (attr % w) * font_size_w;
				int sy = (attr / w) * font_size_h;

				rect.x = sx;
				rect.y = sy;

				area.x = x;
				area.y = y;

				SDL_BlitSurface(console.surface, &rect, screen, &area);
			}
		}
	}
}

void font_frame(int x, int y, int w, int h) {

	console.frame_x = x;
	console.frame_y = y;
	console.frame_w = w;
	console.frame_h = h;
}

void font_locate(int x, int y) {

	console.font_x = x;
	console.font_y = y;
}

void font_printf(const char *format, ...) {

	char *str = strbuf;
	char ch;

	va_list ap;

	va_start(ap, format);

	vsnprintf(strbuf,
			  sizeof(char) * font_map_w * font_map_h,
			  format, ap);

	va_end(ap);

	while (console.font_y < console.frame_h && (ch = *str++) != '\0') {

		switch (ch) {
		case ' ':
			console.map[
				(console.frame_x + console.font_x)
				+ (console.frame_y + console.font_y) * font_map_w] = 0;
			console.font_x++;
			break;
		case '\t':
			console.font_x += 4;
			break;
		case '\n':
			console.font_x = 0;
			console.font_y++;
			break;
		default:
			console.map[
				(console.frame_x + console.font_x)
				+ (console.frame_y + console.font_y) * font_map_w] = ch;
			console.font_x++;
			break;
		}

		if (console.font_x >= console.frame_w) {
			console.font_x = 0;
			console.font_y++;
		}
	}
}

void font_clean(void) {

	int x, y;

	console.font_x = 0;
	console.font_y = 0;

	for (y = 0; y < console.frame_h; y++) {

		unsigned char *mp =
			&console.map[
				(console.frame_y + y) * font_map_w + console.frame_x];

		for (x = 0; x < console.frame_w; x++) {
			*mp++ = 0;
		}
	}
}
