/*
 * sound.c
 *
 * Last modified: <2009/06/24 20:23:11 +0900> By SUMIDA
 */
#include <stdlib.h>
#include <SDL.h>
#include <SDL_mixer.h>
#include "sound.h"
#include "log.h"

/*#define USE_AUDIO /**/

#define CHANNEL_MAX      16

#ifdef USE_AUDIO
static char *music_name;
static Mix_Music *music;
static int channel_no;
static int music_no;
static Mix_Chunk **chunk;
#else
static int dummy_play_count = 0;
#endif

static int max_music = 0;
static int max_chunk = 0;

/*  */
void sound_init(int max_bgm, int max_se) {
#ifdef USE_AUDIO
	REPORT("sound_init ... start.");
	music = NULL;
	music_no = -1;
	channel_no = 0;

	max_music = max_bgm;
	max_chunk = max_se;

	music_name = malloc(sizeof(char) * _MAX_PATH * max_music);
	chunk = malloc(sizeof(Mix_Chunk*) * max_chunk);

	memset(music_name, 0, sizeof(char) * _MAX_PATH * max_music);
	memset(chunk, 0, sizeof(Mix_Chunk*) * max_chunk);

	Mix_AllocateChannels(CHANNEL_MAX);

	REPORT("sound_init ... done.");
#endif
}

void sound_load_music(const char* file, int bgm_id) {
#ifdef USE_AUDIO
	REPORT(format("music_load(%s)", file));
	snprintf(&music_name[_MAX_PATH * bgm_id], _MAX_PATH, "%s", file);
#endif
}

void sound_load_effect(const char* file, int se_id) {
#ifdef USE_AUDIO
	REPORT(format("effect_load(%s)", file));
	chunk[se_id] = Mix_LoadWAV(file);
#endif
}

/* I */
void sound_term(void) {
#ifdef USE_AUDIO
	int i;

	REPORT("sound_term ... start.");

	/* t~ */
	sound_stop_music();

	/* ʉ̉ */
	for (i = 0; i < max_chunk; i++) {
		if (chunk[i] != NULL) {
			Mix_FreeChunk(chunk[i]);
		}
	}

	Mix_CloseAudio();

	REPORT("sound_term ... done.");
#endif
}

void sound_music(int bgm_id, int loop) {
#ifdef USE_AUDIO
	if (music_no != bgm_id) {
		/* REPORT(format("sound_music(%s, %d) ... ", id, loop)); */
		sound_stop_music();
		music = Mix_LoadMUS(&music_name[_MAX_PATH * bgm_id]);
		music_no = bgm_id;
		Mix_PlayMusic(music, loop);
	}
#else
	dummy_play_count = 0;
#endif
}

void sound_stop_music(void) {
#ifdef USE_AUDIO
	/* REPORT("sound_stop_music ... start."); */
	if (music != NULL) {
		Mix_HaltMusic();
		Mix_FreeMusic(music);
		music = NULL;
		music_no = -1;
	}
	/* REPORT("sound_stop_music ... done."); */
#endif
}

void sound_effect(int se_id) {
#ifdef USE_AUDIO
	/* REPORT(format("sound_effect(%d)", se_id)); */
	Mix_PlayChannel(-1, chunk[se_id], 0);
#endif
}

void sound_stop_effect(void) {
#ifdef USE_AUDIO
	/* REPORT("sound_stop_effect ... start."); */
	Mix_HaltChannel(-1);
	channel_no = 0;
	/* REPORT("sound_stop_effect ... done."); */
#endif
}

int sound_playing_music(void) {
#ifdef USE_AUDIO
	return (music != NULL && Mix_PlayingMusic());
#else
	/* REPORT(format("dummy_play_count=%d", dummy_play_count)); */
	return (++dummy_play_count < 10 * 60);
#endif
}
